uplevel #0 {
	global	auto_path env;

	if {[lsearch -exact $auto_path [file dirname [info script]]] < 0} {
		lappend auto_path [file dirname [info script]];
	}

	set {} [file tail [file rootname [info script]]];
	
	while {[file rootname ${}] != ${}} {
		set {} [file rootname ${}];
	}

	package provide ${} 1.0;

	namespace eval ${} {
		variable !includePath;

		# proc to go through the search path of include files
		# to source a given file
		#
		# !! you can use glob pattern in the scriptFileName !!
		#
		proc !include {scriptFileName} {
			variable !includePath;

			if {([file dirname $scriptFileName] == ".") &&
				 [file exists $scriptFileName] && [file isfile $scriptFileName] && [file readable $scriptFileName]} {
				if {[catch {uplevel source $scriptFileName;} reason]} {
					error $reason;
				}

				return $reason;
			}
			
			if {${!includePath} == ""} {
				!reset
			}

			foreach directory ${!includePath} {
				set filePattern [file join $directory $scriptFileName];
				set fileNames   [glob -nocomplain $filePattern];
				set fileName    [lindex $fileNames 0];

				switch -exact -- [llength $fileNames] {
					0  {}
					1  {
						if {[file exists $fileName] && [file isfile $fileName] && [file readable $fileName]} {
							if {[catch {uplevel source $fileName;} reason]} {
								error $reason;
							}

							return $reason;
						}

						error "couldn't include \"$fileName\": no such file or file not readable";
					}
					default  {
						error "couldn't include \"$fileName\": too many files matching \"$scriptFileName\"";
					}
				}
			}

			error "script file \"$scriptFileName\" not found, check your path";
		}

		# build path from variables or given directories
		# e.g. to be used with variables like auto_path,
		# tcl_library, or if existent tk_library
		#
		# !! you can use glob pattern in directories !!
		#
		proc !path {{option {}} args} {
			variable !includePath;

			if {$option == {}} {
				return ${!includePath};
			} elseif {($option != "append") && ($option != "delete")} {
				error "bad option \"$option\": must be append, or delete";
			}

			foreach arg $args {
				if {[uplevel [list info exists $arg]]} {
					!path $option [uplevel [list set $arg]];
				} elseif {[uplevel #0 [list info exists $arg]]} {
					!path $option [uplevel #0 [list set $arg]];
				} else {
					foreach directory $arg {
						switch -exact -- $option {
							append	{
								if {[lsearch -exact ${!includePath} $directory] < 0} {
									if {![file exists $directory] || ![file isdirectory $directory] || ![file readable $directory]} {
										error "couldn't extend path with \"$directory\": no such directory or directory not readable";
									}

									lappend !includePath $directory;
								}
							}
							delete	{
								while {[set idx [lsearch -glob ${!includePath} $directory]] >= 0} {
									set !includePath	[lreplace ${!includePath} $idx $idx];
								}
							}
						}
					}
				}
			}

			return ${!includePath};
		}

		proc !reset {} {
			variable !includePath;

			set !includePath	[pwd];

			return ${!includePath};
		}

		!reset;

		namespace export -clear !*;
	}

	namespace import -force ${}::*;

	catch {unset {}};
}

